@@ -4,20 +4,25 @@
 #' the regression model per group. This helps to inspect the precision
 #' of the modeling process. The scores should not deviate too far from
 #' regression line.
-#' @param data The raw data within a data.frame
-#' @param model The regression model
+#' @param data The raw data within a data.frame or cnorm object
+#' @param model The regression model (optional)
 #' @param group The grouping variable
 #' @param raw The raw score variable
 #' @param type Type of display: 0 = plot manifest against fitted values, 1 = plot
 #' manifest against difference values
 #' @examples
-#' # Load example data set, compute model and plot results
-#' normData <- prepareData()
-#' m <- bestModel(data = normData)
-#' plotRaw(normData, m, group="group")
+#' # Compute model with example dataset and plot results
+#' result <- cnorm(raw = elfe$raw, group = elfe$group)
+#' plotRaw(result)
 #' @export
+#' @family plot
 plotRaw <- function(data, model, group = NULL, raw = NULL, type = 0) {
 
+  if(class(data)=="cnorm"){
+    model <- data$model
+    data <- data$data
+  }
+
   if (!attr(data, "useAge")){
     stop("Age or group variable explicitely set to FALSE in dataset. No plotting available.")
   }
@@ -111,8 +116,8 @@ plotRaw <- function(data, model, group = NULL, raw = NULL, type = 0) {
 #' of the modeling process. The scores should not deviate too far from
 #' regression line. The computation of the standard error is based on Oosterhuis, van der
 #' Ark and Sijtsma (2016).
-#' @param data The raw data within a data.frame
-#' @param model The regression model
+#' @param data The raw data within a data.frame or a cnorm object
+#' @param model The regression model (optional)
 #' @param group The grouping variable, use empty string for no group
 #' @param minNorm lower bound of fitted norm scores
 #' @param maxNorm upper bound of fitted norm scores
@@ -122,12 +127,17 @@ plotRaw <- function(data, model, group = NULL, raw = NULL, type = 0) {
 #' @examples
 #' # Load example data set, compute model and plot results
 #' \dontrun{
-#' normData <- prepareData()
-#' m <- bestModel(data = normData)
-#' plotNorm(normData, m, group="group", minNorm=25, maxNorm=75)
+#' result <- cnorm(raw = elfe$raw, group = elfe$group)
+#' plotNorm(result, group="group", minNorm=25, maxNorm=75)
 #' }
 #' @export
+#' @family plot
 plotNorm <- function(data, model, group = "", minNorm = NULL, maxNorm = NULL, type = 0) {
+  if(class(data)=="cnorm"){
+    model <- data$model
+    data <- data$data
+  }
+
   if (!attr(data, "useAge")){
     stop("Age or group variable explicitely set to FALSE in dataset. No plotting available.")
   }
@@ -230,9 +240,8 @@ plotNorm <- function(data, model, group = "", minNorm = NULL, maxNorm = NULL, ty
 #' The function plots the norm curves based on the regression model.
 #' Please check the function for inconsistent curves: The different
 #' curves should not intersect. Violations of this assumption are a strong
-#' indication for problems
-#' in modeling the relationship between raw and norm scores. There are
-#' several reasons, why this might occur:
+#' indication for violations of model assumptions in modeling the relationship between raw
+#' and norm scores. There are several reasons, why this might occur:
 #' \enumerate{
 #'   \item Vertical extrapolation: Choosing extreme norm scores, e. g. scores
 #'   -3 <= x and x >= 3 In order to model these extreme scores, a large sample
@@ -247,7 +256,7 @@ plotNorm <- function(data, model, group = "", minNorm = NULL, maxNorm = NULL, ty
 #' certain degree outside the original sample, but it should in general
 #' be handled with caution.
 #' checkConsistency and derivationPlot can be used to further inspect the model.
-#' @param model The model from the bestModel function
+#' @param model The model from the bestModel function or a cnorm object
 #' @param normList Vector with norm scores to display
 #' @param minAge Age to start with checking
 #' @param maxAge Upper end of the age check
@@ -261,10 +270,11 @@ plotNorm <- function(data, model, group = "", minNorm = NULL, maxNorm = NULL, ty
 #' @seealso checkConsistency, derivationPlot, plotPercentiles
 #' @examples
 #' # Load example data set, compute model and plot results
-#' normData <- prepareData()
+#' normData <- prepareData(elfe)
 #' m <- bestModel(data = normData)
 #' plotNormCurves(m, minAge=2, maxAge=5)
 #' @export
+#' @family plot
 plotNormCurves <- function(model, normList = c(30, 40, 50, 60, 70),
                            minAge = NULL,
                            maxAge = NULL,
@@ -273,6 +283,10 @@ plotNormCurves <- function(model, normList = c(30, 40, 50, 60, 70),
                            maxRaw = NULL,
                            covariate = NULL) {
 
+  if(class(model)=="cnorm"){
+    model <- model$model
+  }
+
   if(!is.null(covariate)&&is.null(model$covariate)){
     warning("Covariate specified but no covariate available in the model. Setting covariate to NULL.")
     covariate = NULL
@@ -368,8 +382,8 @@ plotNormCurves <- function(model, normList = c(30, 40, 50, 60, 70),
 #' and change the 'type' parameter accordingly.
 #' In case, you get 'jagged' or disorganized percentile curve, try to reduce the 'k'
 #' parameter in modelling.
-#' @param data The raw data including the percentiles and norm scores
-#' @param model The model from the bestModel function
+#' @param data The raw data including the percentiles and norm scores or a cnorm object
+#' @param model The model from the bestModel function (optional)
 #' @param minRaw Lower bound of the raw score (default = 0)
 #' @param maxRaw Upper bound of the raw score
 #' @param minAge Variable to restrict the lower bound of the plot to a specific age
@@ -390,10 +404,10 @@ plotNormCurves <- function(model, normList = c(30, 40, 50, 60, 70),
 #' @seealso plotNormCurves, plotPercentileSeries
 #' @examples
 #' # Load example data set, compute model and plot results
-#' normData <- prepareData()
-#' m <- bestModel(data = normData)
-#' plotPercentiles(normData, m)
+#' result <- cnorm(raw = elfe$raw, group = elfe$group)
+#' plotPercentiles(result)
 #' @export
+#' @family plot
 plotPercentiles <- function(data,
                             model,
                             minRaw = NULL,
@@ -407,6 +421,11 @@ plotPercentiles <- function(data,
                             type = 7,
                             title = NULL, covariate = NULL) {
 
+  if(class(data)=="cnorm"){
+    model <- data$model
+    data <- data$data
+  }
+
   if(!is.null(covariate)&&is.null(model$covariate)){
     warning("Covariate specified but no covariate available in the model. Setting covariate to NULL.")
     covariate = NULL
@@ -427,7 +446,7 @@ plotPercentiles <- function(data,
               minAge = minAge, maxAge = maxAge,
               raw = raw, group = group, percentiles = percentiles,
               scale = scale, title = title))
-    return(print(trel))
+    return(base::print(trel))
   }
 
   if(!is.null(model$covariate)){
@@ -594,7 +613,7 @@ plotPercentiles <- function(data,
     title <- paste0("Observed and Predicted Percentile Curves\nModel: ", model$ideal.model, ", R2 = ", round(model$subsets$adjr2[[model$ideal.model]], digits = 4))
   }
 
-  plot <- xyplot(formula(xyFunction), percentile,
+  chart <- xyplot(formula(xyFunction), percentile,
     panel = function(...)
       panel.superpose(..., panel.groups = panelfun),
     main = title,
@@ -607,8 +626,8 @@ plotPercentiles <- function(data,
     )
   )
 
-  print(plot)
-  return(plot)
+  base::print(chart)
+  return(chart)
 }
 
 
@@ -616,8 +635,9 @@ plotPercentiles <- function(data,
 #'
 #' The function plots the density  curves based on the regression model against
 #' the actual percentiles from the raw data. As in 'plotNormCurves',
-#' please check for inconsistent curves, especially curves showing implausible shapes.
-#' @param model The model from the bestModel function
+#' please check for inconsistent curves, especially curves showing implausible shapes as f. e.
+#' violations of biuniqueness.
+#' @param model The model from the bestModel function or a cnorm object
 #' @param minRaw Lower bound of the raw score
 #' @param maxRaw Upper bound of the raw score
 #' @param minNorm Lower bound of the norm score
@@ -628,16 +648,21 @@ plotPercentiles <- function(data,
 #' @seealso plotNormCurves, plotPercentiles
 #' @examples
 #' # Load example data set, compute model and plot results for age values 2, 4 and 6
-#' normData <- prepareData()
-#' m <- bestModel(data = normData)
-#' plotDensity(m, group = c (2, 4, 6))
+#' result <- cnorm(raw = elfe$raw, group = elfe$group)
+#' plotDensity(result, group = c (2, 4, 6))
 #' @export
+#' @family plot
 plotDensity <- function(model,
                         minRaw = NULL,
                         maxRaw = NULL,
                         minNorm = NULL,
                         maxNorm = NULL,
                         group = NULL, covariate = NULL) {
+
+  if(class(model)=="cnorm"){
+    model <- model$model
+  }
+
   if(!is.null(covariate)&&is.null(model$covariate)){
     warning("Covariate specified but no covariate available in the model. Setting covariate to NULL.")
     covariate = NULL
@@ -697,7 +722,7 @@ plotDensity <- function(model,
     data = matrix, groups = group,
     panel = function(...)
       panel.superpose(..., panel.groups = panelfun),
-    main = "Density function",
+    main = "Density functions",
     ylab = "Density", xlab = "Raw Score",
     col = COL, lwd = 1.5, grid = TRUE,
     key = list(
@@ -706,7 +731,7 @@ plotDensity <- function(model,
       text = list(NAMES)
     )
   )
-  print(plot)
+  base::print(plot)
   return(matrix)
 }
 
@@ -718,8 +743,8 @@ plotDensity <- function(model,
 #' to determine the bounds of the modeling (age and standard score range). It can be used as an
 #' additional model check to determine the best fitting model. Please have a look at the
 #'' plotPercentiles' function for further information.
-#' @param data The raw data including the percentiles and norm scores
-#' @param model The model from the bestModel function
+#' @param data The raw data including the percentiles and norm scores or a cnorm object
+#' @param model The model from the bestModel function (optional)
 #' @param start Number of predictors to start with
 #' @param end Number of predictors to end with
 #' @param group The name of the grouping variable; the distinct groups are automatically
@@ -735,17 +760,24 @@ plotDensity <- function(model,
 #'
 #' @examples
 #' # Load example data set, compute model and plot results
-#' normData <- prepareData(elfe)
-#' model <- bestModel(data = normData)
-#' plotPercentileSeries(normData, model, start=1, end=5, group="group")
+#' result <- cnorm(raw = elfe$raw, group = elfe$group)
+#' plotPercentileSeries(result, start=1, end=5, group="group")
+#' @family plot
 plotPercentileSeries <- function(data, model, start = 1, end = NULL, group = NULL,
                                  percentiles = c(0.025, 0.1, 0.25, 0.5, 0.75, 0.9, 0.975),
                                  type = 7,
                                  filename = NULL) {
+  if(class(data)=="cnorm"){
+    model <- data$model
+    d <- data$data
+  }else{
+    d <- as.data.frame(data)
+  }
+
   if(!is.null(model$covariate)){
     stop("This function us currently not able to handle models with covariates.")
   }
-  d <- as.data.frame(data)
+
 
   if (!attr(d, "useAge")){
     stop("Age or group variable explicitely set to FALSE in dataset. No plotting available.")
@@ -821,7 +853,7 @@ plotPercentileSeries <- function(data, model, start = 1, end = NULL, group = NUL
 
     if (!is.null(filename)) {
       trellis.device(device = "png", filename = paste0(filename, start, ".png"))
-      print(l[[length(l)]])
+      base::print(l[[length(l)]])
       dev.off()
     }
     start <- start + 1
@@ -844,7 +876,7 @@ plotPercentileSeries <- function(data, model, start = 1, end = NULL, group = NUL
 #' In the plot, Mallow's Cp is log transformed and the BIC is always highly
 #' negative. The R2 cutoff that was specified in the bestModel function is
 #' displayed as a dashed line.
-#' @param model The regression model from the bestModel function
+#' @param model The regression model from the bestModel function or a cnorm object
 #' @param type Type of chart with 0 = adjusted R2 by number of predictors,
 #' 1 = log transformed Mallow's Cp by adjusted R2, 2 = Bayesian Information
 #' Criterion (BIC) by adjusted R2 and 3 = Root Mean Square Error (RMSE) by number
@@ -852,11 +884,17 @@ plotPercentileSeries <- function(data, model, start = 1, end = NULL, group = NUL
 #' @param index add index labels to data points
 #' @seealso bestModel, plotPercentiles, printSubset
 #' @examples
-#' normData <- prepareData()
-#' m <- bestModel(data = normData)
-#' plotSubset(m)
+#' # Compute model with example data and plot information function
+#' cnorm.model <- cnorm(raw = elfe$raw, group = elfe$group)
+#' plotSubset(cnorm.model)
 #' @export
-plotSubset <- function(model, type = 1, index = FALSE) {
+#' @family plot
+plotSubset <- function(model, type = 0, index = FALSE) {
+
+  if(class(model)=="cnorm"){
+    model <- model$model
+  }
+
   dataFrameTMP <- data.frame(adjr2 = model$subsets$adjr2, bic = model$subsets$bic, cp = model$subsets$cp, RMSE = sqrt(model$subsets$rss / length(model$fitted.values)), nr = seq(1, length(model$subsets$adjr2), by = 1))
   indexLabel <- seq(from = 1, to = nrow(dataFrameTMP))
 
@@ -976,15 +1014,14 @@ plotSubset <- function(model, type = 1, index = FALSE) {
 #' norm scores within each age group. The regression based modeling approach
 #' relies on the assumption of a linear progression of the norm scores.
 #' Negative scores in the first order derivative indicate a violation of this
-#' assumption. Scores near zero
-#' are typical for bottom and ceiling effects in the raw data.
+#' assumption. Scores near zero are typical for bottom and ceiling effects in the raw data.
 #' The regression models usually converge within the range of the original
 #' values. In case of vertical and horizontal extrapolation, with increasing
 #' distance to the original data, the risk of assumption violation increases
 #' as well.
 #' ATTENTION: plotDerivative is currently still incompatible with reversed raw
 #' score scales ('descent' option)
-#' @param model The model from the bestModel function
+#' @param model The model from the bestModel function or a cnorm object
 #' @param minAge Age to start with checking
 #' @param maxAge Upper end of the age check
 #' @param stepAge Stepping parameter for the age check, usually 1 or 0.1; lower
@@ -997,10 +1034,10 @@ plotSubset <- function(model, type = 1, index = FALSE) {
 #' @seealso checkConsistency, bestModel, derive
 #' @examples
 #' # Load example data set, compute model and plot results
-#' normData <- prepareData()
-#' m <- bestModel(data = normData)
-#' plotDerivative(m, minAge=2, maxAge=5, step=.2, minNorm=25, maxNorm=75, stepNorm=1)
+#' result <- cnorm(raw = elfe$raw, group = elfe$group)
+#' plotDerivative(result, minAge=2, maxAge=5, step=.2, minNorm=25, maxNorm=75, stepNorm=1)
 #' @export
+#' @family plot
 plotDerivative <- function(model,
                            minAge = NULL,
                            maxAge = NULL,
@@ -1009,10 +1046,16 @@ plotDerivative <- function(model,
                            stepAge = 0.2,
                            stepNorm = 1,
                            order = 1) {
+
+  if(class(model)=="cnorm"){
+    model <- model$model
+  }
+
   if (!model$useAge){
     stop("Age or group variable explicitely set to FALSE in dataset. No plotting available.")
   }
 
+
   if (is.null(minAge)) {
     minAge <- model$minA1
   }
@@ -1033,7 +1076,7 @@ plotDerivative <- function(model,
   colS <- c(seq(minAge, maxAge, length.out = 1 + (maxAge - minAge) / stepAge))
   coeff <- derive(model, order)
   cat(paste0(rangeCheck(model, minAge, maxAge, minNorm, maxNorm), " Coefficients from the ", order, " order derivative function:\n\n"))
-  print(coeff)
+  base::print(coeff)
 
   devFrame <- data.frame(matrix(NA, nrow = length(rowS), ncol = length(colS)))
   dev2 <- data.frame()
@@ -1103,3 +1146,39 @@ plotDerivative <- function(model,
   p3 <- p1 + p2
   p3
 }
+
+#' General convencience plotting function
+#'
+#' @param x a cnorm object
+#' @param y the type of plot as a string, can be one of
+#' 'raw', 'norm', 'curves', 'percentiles', 'series', 'subset', or 'derivative'
+#' @param ... additional parameters for the specific plotting function
+#'
+#' @export
+plotCnorm <- function(x, y, ...){
+  if(!class(x)=="cnorm"||!is.character(y)){
+    message("Please provide a cnorm object as parameter x and the type of plot as a string for parameter y, which can be 'raw', 'norm', 'curves', 'percentiles', 'series', 'subset', or 'derivative'.")
+    return()
+  }
+
+  if(y == "raw"){
+    plotRaw(x, ...)
+  }else if(y == "norm"){
+    plotNorm(x, ...)
+  }else if(y == "curves"){
+    plotNormCurves(x, ...)
+  }else if(y == "percentiles"){
+    plotPercentiles(x, ...)
+  }else if(y == "density"){
+    plotDensity(x, ...)
+  }else if(y == "series"){
+    plotPercentileSeries(x, ...)
+  }else if(y == "subset"){
+    plotSubset(x, ...)
+  }else if(y == "derivative"){
+    plotDerivative(x, ...)
+  }else{
+    message("Please provide the type of plot as a string for parameter y, which can be 'raw', 'norm', 'curves', 'percentiles', 'series', 'subset', or 'derivative'.")
+    return()
+  }
+}
